@@ -3,7 +3,7 @@
 # This file is part of Hypothesis, which may be found at
 # https://github.com/HypothesisWorks/hypothesis-python
 #
-# Most of this work is copyright (C) 2013-2016 David R. MacIver
+# Most of this work is copyright (C) 2013-2017 David R. MacIver
 # (david@drmaciver.com), but it contains contributions by others. See
 # CONTRIBUTING.rst for a full list of people who may hold copyright, and
 # consult the git log if you need to determine who owns an individual
@@ -28,12 +28,12 @@ import os
 import inspect
 import warnings
 import threading
-from enum import Enum, unique, IntEnum
+from enum import Enum, IntEnum, unique
 from collections import namedtuple
 
 from hypothesis.errors import InvalidArgument, HypothesisDeprecationWarning
 from hypothesis.configuration import hypothesis_home_dir
-from hypothesis.utils.conventions import not_set
+from hypothesis.utils.conventions import UniqueIdentifier, not_set
 from hypothesis.utils.dynamicvariables import DynamicVariable
 
 __all__ = [
@@ -41,6 +41,9 @@ __all__ = [
 ]
 
 
+unlimited = UniqueIdentifier('unlimited')
+
+
 all_settings = {}
 
 
@@ -49,8 +52,9 @@ _db_cache = {}
 
 class settingsProperty(object):
 
-    def __init__(self, name):
+    def __init__(self, name, show_default):
         self.name = name
+        self.show_default = show_default
 
     def __get__(self, obj, type=None):
         if obj is None:
@@ -69,9 +73,11 @@ class settingsProperty(object):
 
     @property
     def __doc__(self):
+        default = repr(getattr(settings.default, self.name)) if \
+            self.show_default else '(dynamically calculated)'
         return '\n'.join((
             all_settings[self.name].description,
-            'default value: %r' % (getattr(settings.default, self.name),)
+            'default value: %s' % (default,)
         ))
 
 
@@ -135,14 +141,21 @@ class settings(settingsMeta('settings', (object,), {})):
         self._construction_complete = False
         self._database = kwargs.pop('database', not_set)
         database_file = kwargs.get('database_file', not_set)
+        deprecations = []
         defaults = parent or settings.default
         if defaults is not None:
             for setting in all_settings.values():
                 if kwargs.get(setting.name, not_set) is not_set:
                     kwargs[setting.name] = getattr(defaults, setting.name)
-                elif setting.validator:
-                    kwargs[setting.name] = setting.validator(
-                        kwargs[setting.name])
+                else:
+                    if kwargs[setting.name] != setting.future_default:
+                        if (
+                            setting.deprecation_message is not None
+                        ):
+                            deprecations.append(setting)
+                    if setting.validator:
+                        kwargs[setting.name] = setting.validator(
+                            kwargs[setting.name])
             if self._database is not_set and database_file is not_set:
                 self._database = defaults.database
         for name, value in kwargs.items():
@@ -153,6 +166,9 @@ class settings(settingsMeta('settings', (object,), {})):
         self.storage = threading.local()
         self._construction_complete = True
 
+        for d in deprecations:
+            note_deprecation(d.deprecation_message, self)
+
     def defaults_stack(self):
         try:
             return self.storage.defaults_stack
@@ -167,7 +183,8 @@ class settings(settingsMeta('settings', (object,), {})):
     @classmethod
     def define_setting(
         cls, name, description, default, options=None, deprecation=None,
-        validator=None,
+        validator=None, show_default=True, future_default=not_set,
+        deprecation_message=None,
     ):
         """Add a new setting.
 
@@ -188,10 +205,14 @@ class settings(settingsMeta('settings', (object,), {})):
             options = tuple(options)
             assert default in options
 
+        if future_default is not_set:
+            future_default = default
+
         all_settings[name] = Setting(
-            name, description.strip(), default, options, validator
+            name, description.strip(), default, options, validator,
+            future_default, deprecation_message,
         )
-        setattr(settings, name, settingsProperty(name))
+        setattr(settings, name, settingsProperty(name, show_default))
 
     @classmethod
     def lock_further_definitions(cls):
@@ -243,8 +264,8 @@ class settings(settingsMeta('settings', (object,), {})):
         If this was explicitly set at settings instantiation then that
         value will be used (even if it was None). If not and the
         database_file setting is not None this will be lazily loaded as
-        an SQLite backed ExampleDatabase using that file the first time
-        this property is accessed on a particular thread.
+        an ExampleDatabase using that file the first time this property
+        is accessed on a particular thread.
 
         """
         if self._database is not_set and self.database_file is not None:
@@ -312,7 +333,8 @@ class settings(settingsMeta('settings', (object,), {})):
 
 Setting = namedtuple(
     'Setting', (
-        'name', 'description', 'default', 'options', 'validator'
+        'name', 'description', 'default', 'options', 'validator',
+        'future_default', 'deprecation_message',
     ))
 
 
@@ -377,6 +399,14 @@ shrink the example.
 """
 )
 
+
+def _validate_timeout(n):
+    if n is unlimited:
+        return -1
+    else:
+        return n
+
+
 settings.define_setting(
     'timeout',
     default=60,
@@ -386,7 +416,18 @@ if it has not found many examples. This is a soft rather than a hard
 limit - Hypothesis won't e.g. interrupt execution of the called
 function to stop it. If this value is <= 0 then no timeout will be
 applied.
-"""
+
+Note: This setting is deprecated. In future Hypothesis will be removing the
+timeout feature.
+""",
+    deprecation_message="""
+The timeout feature will go away in a future version of Hypothesis. To get
+the future behaviour set timeout=hypothesis.unlimited instead (which will
+remain valid for a further deprecation period after this setting has gone
+away).
+""",
+    future_default=unlimited,
+    validator=_validate_timeout
 )
 
 settings.define_setting(
@@ -423,6 +464,7 @@ settings.define_setting(
         os.getenv('HYPOTHESIS_DATABASE_FILE') or
         os.path.join(hypothesis_home_dir(), 'examples')
     ),
+    show_default=False,
     description="""
     database: An instance of hypothesis.database.ExampleDatabase that will be
 used to save examples to and load previous examples from. May be None
@@ -447,6 +489,7 @@ class HealthCheck(Enum):
     too_slow = 3
     random_module = 4
     return_value = 5
+    hung_test = 6
 
 
 @unique
