@@ -26,17 +26,21 @@ import traceback
 from random import Random
 from collections import namedtuple
 
+import hypothesis.strategies as st
 from hypothesis.errors import Flaky, Timeout, NoSuchExample, \
     Unsatisfiable, InvalidArgument, FailedHealthCheck, \
     UnsatisfiedAssumption, HypothesisDeprecationWarning
 from hypothesis.control import BuildContext
 from hypothesis._settings import settings as Settings
-from hypothesis._settings import Phase, Verbosity, HealthCheck
+from hypothesis._settings import Phase, Verbosity, HealthCheck, \
+    note_deprecation
 from hypothesis.executors import new_style_executor, \
     default_new_style_executor
 from hypothesis.reporting import report, verbose_report, current_verbosity
 from hypothesis.statistics import note_engine_for_statistics
-from hypothesis.internal.compat import str_to_bytes, getfullargspec
+from hypothesis.internal.compat import str_to_bytes, get_type_hints, \
+    getfullargspec
+from hypothesis.utils.conventions import infer
 from hypothesis.internal.escalation import \
     escalate_hypothesis_internal_error
 from hypothesis.internal.reflection import nicerepr, arg_string, \
@@ -153,22 +157,25 @@ def is_invalid_test(
         return invalid(
             'given must be called with at least one argument')
 
-    if (generator_arguments and (original_argspec.varargs or
-                                 original_argspec.varkw)):
+    if generator_arguments and any([original_argspec.varargs,
+                                    original_argspec.varkw,
+                                    original_argspec.kwonlyargs]):
         return invalid(
-            'varargs or keywords are not supported with positional '
-            'arguments to @given'
+            'positional arguments to @given are not supported with varargs, '
+            'varkeywords, or keyword-only arguments'
         )
 
-    if (
-        len(generator_arguments) > len(original_argspec.args)
-    ):
+    if len(generator_arguments) > len(original_argspec.args):
         return invalid((
             'Too many positional arguments for %s() (got %d but'
             ' expected at most %d') % (
                 name, len(generator_arguments),
                 len(original_argspec.args)))
 
+    if infer in generator_arguments:
+        return invalid('infer was passed as a positional argument to @given, '
+                       'but may only be passed as a keyword argument')
+
     if generator_arguments and generator_kwargs:
         return invalid(
             'cannot mix positional and keyword arguments to @given'
@@ -226,18 +233,20 @@ def execute_explicit_examples(
         # Note: Test may mutate arguments and we can't rerun explicit
         # examples, so we have to calculate the failure message at this
         # point rather than than later.
-        message_on_failure = 'Falsifying example: %s(%s)' % (
+        example_string = '%s(%s)' % (
             test.__name__, arg_string(test, arguments, example_kwargs)
         )
         try:
             with BuildContext(None) as b:
+                if settings.verbosity >= Verbosity.verbose:
+                    report('Trying example: ' + example_string)
                 test_runner(
                     None,
                     lambda data: test(*arguments, **example_kwargs)
                 )
         except BaseException:
             traceback.print_exc()
-            report(message_on_failure)
+            report('Falsifying example: ' + example_string)
             for n in b.notes:
                 report(n)
             raise
@@ -247,8 +256,11 @@ def fail_health_check(settings, message, label):
     # Tell pytest to omit the body of this function from tracebacks
     # http://doc.pytest.org/en/latest/example/simple.html#writing-well-integrated-assertion-helpers
     __tracebackhide__ = True
+
     if label in settings.suppress_health_check:
         return
+    if not settings.perform_health_check:
+        return
     message += (
         '\nSee https://hypothesis.readthedocs.io/en/latest/health'
         'checks.html for more information about this. '
@@ -403,8 +415,6 @@ def process_arguments_to_given(
     wrapped_test, arguments, kwargs, generator_arguments, generator_kwargs,
     argspec, test, settings
 ):
-    import hypothesis.strategies as sd
-
     selfy = None
     arguments, kwargs = convert_positional_arguments(
         wrapped_test, arguments, kwargs)
@@ -420,14 +430,13 @@ def process_arguments_to_given(
 
     arguments = tuple(arguments)
 
-    given_specifier = sd.tuples(
-        sd.just(arguments),
-        sd.fixed_dictionaries(generator_kwargs).map(
+    search_strategy = st.tuples(
+        st.just(arguments),
+        st.fixed_dictionaries(generator_kwargs).map(
             lambda args: dict(args, **kwargs)
         )
     )
 
-    search_strategy = given_specifier
     if selfy is not None:
         search_strategy = WithRunner(search_strategy, selfy)
 
@@ -436,6 +445,56 @@ def process_arguments_to_given(
     return arguments, kwargs, test_runner, search_strategy
 
 
+def skip_exceptions_to_reraise():
+    """Return a tuple of exceptions meaning 'skip this test', to re-raise.
+
+    This is intended to cover most common test runners; if you would
+    like another to be added please open an issue or pull request.
+
+    """
+    import unittest
+    # This is a set because nose may simply re-export unittest.SkipTest
+    exceptions = set([unittest.SkipTest])
+
+    try:  # pragma: no cover
+        from unittest2 import SkipTest
+        exceptions.add(SkipTest)
+    except ImportError:
+        pass
+
+    try:  # pragma: no cover
+        from pytest.runner import Skipped
+        exceptions.add(Skipped)
+    except ImportError:
+        pass
+
+    try:  # pragma: no cover
+        from nose import SkipTest as NoseSkipTest
+        exceptions.add(NoseSkipTest)
+    except ImportError:
+        pass
+
+    return tuple(sorted(exceptions, key=str))
+
+
+exceptions_to_reraise = skip_exceptions_to_reraise()
+
+
+def new_given_argspec(original_argspec, generator_kwargs):
+    """Make an updated argspec for the wrapped test."""
+    new_args = [a for a in original_argspec.args if a not in generator_kwargs]
+    new_kwonlyargs = [a for a in original_argspec.kwonlyargs
+                      if a not in generator_kwargs]
+    annots = {k: v for k, v in original_argspec.annotations.items()
+              if k in new_args + new_kwonlyargs}
+    annots['return'] = None
+    return original_argspec._replace(
+        args=new_args, kwonlyargs=new_kwonlyargs, annotations=annots)
+
+
+HUNG_TEST_TIME_LIMIT = 5 * 60
+
+
 class StateForActualGivenExecution(object):
 
     def __init__(self, test_runner, search_strategy, test, settings, random):
@@ -450,6 +509,15 @@ class StateForActualGivenExecution(object):
         self.random = random
 
     def evaluate_test_data(self, data):
+        if (
+            time.time() - self.start_time >= HUNG_TEST_TIME_LIMIT
+        ):
+            fail_health_check(self.settings, (
+                'Your test has been running for at least five minutes. This '
+                'is probably not what you intended, so by default Hypothesis '
+                'turns it into an error.'
+            ), HealthCheck.hung_test)
+
         try:
             result = self.test_runner(data, reify_and_execute(
                 self.search_strategy, self.test,
@@ -466,7 +534,7 @@ class StateForActualGivenExecution(object):
         except (
             HypothesisDeprecationWarning, FailedHealthCheck,
             StopTest,
-        ):
+        ) + exceptions_to_reraise:
             raise
         except Exception:
             escalate_hypothesis_internal_error()
@@ -478,7 +546,7 @@ class StateForActualGivenExecution(object):
         # Tell pytest to omit the body of this function from tracebacks
         __tracebackhide__ = True
         database_key = str_to_bytes(fully_qualified_name(self.test))
-        start_time = time.time()
+        self.start_time = time.time()
         runner = ConjectureRunner(
             self.evaluate_test_data,
             settings=self.settings, random=self.random,
@@ -486,11 +554,8 @@ class StateForActualGivenExecution(object):
         )
         runner.run()
         note_engine_for_statistics(runner)
-        run_time = time.time() - start_time
-        timed_out = (
-            self.settings.timeout > 0 and
-            run_time >= self.settings.timeout
-        )
+        run_time = time.time() - self.start_time
+        timed_out = runner.exit_reason == ExitReason.timeout
         if runner.last_data is None:
             return
         if runner.last_data.status == Status.INTERESTING:
@@ -500,6 +565,19 @@ class StateForActualGivenExecution(object):
                     database_key, self.falsifying_example
                 )
         else:
+            if timed_out:
+                note_deprecation((
+                    'Your tests are hitting the settings timeout (%.2fs). '
+                    'This functionality will go away in a future release '
+                    'and you should not rely on it. Instead, try setting '
+                    'max_examples to be some value lower than %d (the number '
+                    'of examples your test successfully ran here). Or, if you '
+                    'would prefer your tests to run to completion, regardless '
+                    'of how long they take, you can set the timeout value to '
+                    'hypothesis.unlimited.'
+                ) % (
+                    self.settings.timeout, runner.valid_examples),
+                    self.settings)
             if runner.valid_examples < min(
                 self.settings.min_satisfying_examples,
                 self.settings.max_examples,
@@ -592,20 +670,11 @@ def given(*given_arguments, **given_kwargs):
         if check_invalid is not None:
             return check_invalid
 
-        arguments = original_argspec.args
-
-        for name, strategy in zip(arguments[-len(generator_arguments):],
-                                  generator_arguments):
+        for name, strategy in zip(reversed(original_argspec.args),
+                                  reversed(generator_arguments)):
             generator_kwargs[name] = strategy
 
-        argspec = original_argspec._replace(
-            args=[a for a in arguments if a not in generator_kwargs],
-            defaults=None,
-        )
-        annots = {k: v for k, v in argspec.annotations.items()
-                  if k in (argspec.args + argspec.kwonlyargs)}
-        annots['return'] = None
-        argspec = argspec._replace(annotations=annots)
+        argspec = new_given_argspec(original_argspec, generator_kwargs)
 
         @impersonate(test)
         @define_function_signature(
@@ -619,6 +688,16 @@ def given(*given_arguments, **given_kwargs):
 
             random = get_random_for_wrapped_test(test, wrapped_test)
 
+            if infer in generator_kwargs.values():
+                hints = get_type_hints(test)
+            for name in [name for name, value in generator_kwargs.items()
+                         if value is infer]:
+                if name not in hints:
+                    raise InvalidArgument(
+                        'passed %s=infer for %s, but %s has no type annotation'
+                        % (name, test.__name__, name))
+                generator_kwargs[name] = st.from_type(hints[name])
+
             processed_args = process_arguments_to_given(
                 wrapped_test, arguments, kwargs, generator_arguments,
                 generator_kwargs, argspec, test, settings
